﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;

namespace paralelne_nasobenie_matic
{
    // Deklarácia triedy, ktorej inštancie budú 
    // maticami typu 300x300.
    class Matica300x300
    {
        // Definícia dátového člena triedy, 
        // ktorý bude inicializovaný odkazom na dvojrozmerné pole 
        // s dátami matice.
        private int[,] cleny;
        // Definícia verejného bezparametrického 
        // inštančného konštruktora.
        public Matica300x300()
        {
            // Inicializácia matice typu 300x300.
            cleny = new int[300, 300];
            for (int i = 0; i < 300; i++)
            {
                for (int j = 0; j < 300; j++)
                {
                    cleny[i, j] = i + j + 1;
                }
            }
        }
        // Definícia verejnej parametrickej indexovanej 
        // inštančnej vlastnosti.
        public int this[int i, int j]
        {
            get
            {
                return cleny[i, j];
            }
            set
            {
                cleny[i, j] = value;
            }
        }
        // Definícia verejnej bezparametrickej inštančnej metódy 
        // na zobrazenie obsahu matice. 
        public void ZobrazitObsah()
        {
            for (int i = 0; i < 300; i++)
            {
                for (int j = 0; j < 300; j++)
                {
                    Console.Write("{0} ", cleny[i, j]);
                }
                Console.WriteLine();
            }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            const int pocetMatic = 40;
            Matica300x300[] m1 = new Matica300x300[pocetMatic];
            Matica300x300[] m2 = new Matica300x300[pocetMatic];
            Matica300x300[] m3 = new Matica300x300[pocetMatic];
            Matica300x300[] m4 = new Matica300x300[pocetMatic];
            Matica300x300[] m5 = new Matica300x300[pocetMatic];
            Matica300x300[] m6 = new Matica300x300[pocetMatic];

            for (int i = 0; i < pocetMatic; i++)
            {
                m1[i] = new Matica300x300();
                m2[i] = new Matica300x300();
                m3[i] = new Matica300x300();
                m4[i] = new Matica300x300();
                m5[i] = new Matica300x300();
                m6[i] = new Matica300x300();
            }

            Stopwatch stopky = new Stopwatch();
            stopky.Start();

            Console.WriteLine("Paralelné násobenie matíc.");
            // Vytvorenie úlohy, ktorá bude realizovať 
            // paralelné násobenie prvej dvojice matíc typu 300x300.
            Task uloha1 = new Task(() =>
            {
                Console.WriteLine("Úloha 1: Násobenie matíc.");
                Parallel.For(0, pocetMatic, i =>
                {
                    for (int a = 0; a < 300; a++)
                    {
                        for (int b = 0; b < 300; b++)
                        {
                            m5[i][a, b] = 0;
                            for (int k = 0; k < 300; k++)
                            {
                                m5[i][a, b] +=
                                    m1[i][a, k] * m2[i][k, b];
                            }
                        }
                    }
                });
            });
            // Vytvorenie úlohy, ktorá bude realizovať 
            // paralelné násobenie druhej dvojice matíc typu 
            // 300x300.
            Task uloha2 = new Task(() =>
            {
                Console.WriteLine("Úloha 2: Násobenie matíc.");
                Parallel.For(0, pocetMatic, i =>
                {
                    for (int a = 0; a < 300; a++)
                    {
                        for (int b = 0; b < 300; b++)
                        {
                            m6[i][a, b] = 0;
                            for (int k = 0; k < 300; k++)
                            {
                                m6[i][a, b] += 
                                    m3[i][a, k] * m4[i][k, b];
                            }
                        }
                    }
                });
            });
            // Začatie spracovania úloh. 
            uloha1.Start();
            uloha2.Start();
            // Primárne programové vlákno čaká, 
            // až dokiaľ nebudú hotové všetky paralelne 
            // uskutočňované úlohy. 
            Task.WaitAll(uloha1, uloha2);
            stopky.Stop();
            Console.WriteLine("Celkový čas trvania: {0} ms.",
                stopky.ElapsedMilliseconds);
        }
    }
}

